@@ -353,9 +353,9 @@ class Formatter(AbstractFormatter):
         if factory is None:
             factory = lambda f: f or None
         self._factory = factory
-        self._themes = {}
+        self._format_caches = {}
         if theme is not None:
-            self.add_theme(None, theme)
+            self.add_theme(theme)
 
     def format_caches(self):
         """Reimplemented to return the format caches added by add_theme().
@@ -364,17 +364,17 @@ class Formatter(AbstractFormatter):
         enable fast formatting.
 
         """
-        return self._themes
+        return self._format_caches
 
-    def add_theme(self, language, theme, add_baseformat=False):
+    def add_theme(self, theme, language=None, add_baseformat=False):
         """Add a Theme.
 
-        If ``language`` is None, the theme becomes the default theme and the
-        ``add_baseformat`` argument is ignored. If a language is specified (a
-        :class:`~parce.language.Language` subclass), the theme will be used for
-        tokens from that language. If ``add_baseformat`` is True, the theme's
-        baseformat color (window) will be added to all the theme's text
-        formats.
+        If ``language`` is None, the theme becomes the default theme. If a
+        :class:`~parce.language.Language` is specified, the theme will be used
+        for tokens from that language.
+
+        If ``add_baseformat`` is True, the theme's baseformat (window) will be
+        added to all the theme's text formats.
 
         """
         if add_baseformat:
@@ -417,7 +417,7 @@ class Formatter(AbstractFormatter):
         """Copy all themes from the other formatter."""
         self.format_caches().clear()
         for language, fc in formatter.format_caches().items():
-            self.add_theme(language, fc.theme, fc.base is not None)
+            self.add_theme(fc.theme, language, fc.base is not None)
 
 
 class SimpleFormatter(AbstractFormatter):
@@ -441,7 +441,7 @@ class SimpleFormatter(AbstractFormatter):
 
     """
     def format_caches(self):
-        """Reimplemented to return a FormatCache with a factory that concerts
+        """Reimplemented to return a FormatCache with a factory that converts
         an action to a css class string.
 
         """
