@@ -632,9 +632,8 @@ class Context(list, Node):
         """Return a copy of the context, but with the specified parent."""
         # a non-recursive implementation due to Python's recursion limits
         copy = copy_root = type(self)(self.lexicon, parent)
-        stack = []
-        i = 0
         n = self
+        i = 0
         while True:
             z = len(n)
             while i < z:
@@ -642,29 +641,18 @@ class Context(list, Node):
                 if m.is_context:
                     copy.append(type(m)(m.lexicon, copy))
                     copy = copy[-1]
-                    stack.append(i)
                     i = 0
                     n = m
                     break
-                elif m.group is not None:
-                    for g, j in enumerate(range(i + 1, z), m.group + 1):
-                        if n[j].is_context or n[j].group is None or n[j].group < g:
-                            copy.extend(node.copy(copy) for node in n[i:j])
-                            i = j
-                            break
-                    else:
-                        copy.append(m.copy(copy))
-                        i += 1
                 else:
                     copy.append(m.copy(copy))
                     i += 1
             else:
-                if stack:
-                    copy = copy.parent
-                    n = n.parent
-                    i = stack.pop() + 1
-                else:
+                if copy is copy_root:
                     break
+                n = n.parent
+                copy = copy.parent
+                i = len(copy)
         return copy_root
 
     @property
@@ -1043,17 +1031,21 @@ class Context(list, Node):
             yield self, slice(start, None)
 
 
-def make_tokens(event, parent=None):
-    """Factory returning a tuple of one or more Token instances for the event.
+def make_tokens(lexemes, parent=None):
+    """Factory returning a tuple of one or more :class:`Token` instances for
+    the lexemes.
 
-    The event is an Event namedtuple defined in the :mod:`~parce.lexer` module.
-    If the event contains more than one lexeme, GroupToken instances are
-    created.
+    The ``lexemes`` argument is an iterable of three-tuples like the
+    ``lexemes`` in an :class:`~parce.lexer.Event` namedtuple defined in the
+    :mod:`~parce.lexer` module. If there is more than one lexeme,
+    :class:`GroupToken` instances are created.
+
+    The specified ``parent`` context is set as parent, if given.
 
     """
-    if len(event.lexemes) > 1:
-        return GroupToken.make_group(parent, event.lexemes)
+    if len(lexemes) > 1:
+        return GroupToken.make_group(parent, lexemes)
     else:
-        return Token(parent, *event.lexemes[0]),
+        return Token(parent, *lexemes[0]),
 
 
